searchData={"items":[{"type":"module","title":"Cryppo","doc":"Main public API of Cryppo","ref":"Cryppo.html"},{"type":"function","title":"Cryppo.decrypt/2","doc":"Decrypt encrypted data with an encryption key","ref":"Cryppo.html#decrypt/2"},{"type":"function","title":"Example - Cryppo.decrypt/2","doc":"iex> {encrypted_data, encryption_key} = Cryppo.encrypt(\"data to encrypt\", \"Aes256Gcm\")\n    iex> Cryppo.decrypt(encrypted_data, encryption_key)\n    {:ok, \"data to encrypt\"}","ref":"Cryppo.html#decrypt/2-example"},{"type":"function","title":"Cryppo.decrypt_with_derived_key/2","doc":"Decrypt data with a derived key","ref":"Cryppo.html#decrypt_with_derived_key/2"},{"type":"function","title":"Example - Cryppo.decrypt_with_derived_key/2","doc":"iex> encrypted = Cryppo.encrypt_with_derived_key(\"data to encrypt\", \"Aes256Gcm\", \"Pbkdf2Hmac\", \"passphrase\")\n    iex> {:ok, decrypted, _key} = Cryppo.decrypt_with_derived_key(encrypted, \"passphrase\")\n    iex> decrypted\n    \"data to encrypt\"","ref":"Cryppo.html#decrypt_with_derived_key/2-example"},{"type":"function","title":"Cryppo.derivation_strategies/0","doc":"List available  derivation strategies","ref":"Cryppo.html#derivation_strategies/0"},{"type":"function","title":"Cryppo.encrypt/2","doc":"Generate an encryption key for an encryption strategy and encrypt data with this encryption key","ref":"Cryppo.html#encrypt/2"},{"type":"function","title":"Example - Cryppo.encrypt/2","doc":"iex> {_encrypted_data, _encryption_key} = Cryppo.encrypt(\"data to encrypt\", \"Aes256Gcm\")","ref":"Cryppo.html#encrypt/2-example"},{"type":"function","title":"Cryppo.encrypt/3","doc":"Encrypt data with an encryption key","ref":"Cryppo.html#encrypt/3"},{"type":"function","title":"Example - Cryppo.encrypt/3","doc":"iex> encryption_key = Cryppo.generate_encryption_key(\"Aes256Gcm\")\n    iex> _encrypted_data = Cryppo.encrypt(\"data to encrypt\", \"Aes256Gcm\", encryption_key)\n\nThe encryption key must match the encryption strategy:\n\n    iex> encryption_key = Cryppo.generate_encryption_key(\"Aes256Gcm\")\n    iex> Cryppo.encrypt(\"data to encrypt\", \"Rsa4096\", encryption_key)\n    {:incompatible_key, [submitted_key_strategy: Cryppo.Aes256gcm, encryption_strategy: Cryppo.Rsa4096]}","ref":"Cryppo.html#encrypt/3-example"},{"type":"function","title":"Cryppo.encrypt_with_derived_key/4","doc":"Encrypt data with a derived key","ref":"Cryppo.html#encrypt_with_derived_key/4"},{"type":"function","title":"Example - Cryppo.encrypt_with_derived_key/4","doc":"iex> _encrypted = Cryppo.encrypt_with_derived_key(\"data to encrypt\", \"Aes256Gcm\", \"Pbkdf2Hmac\", \"passphrase\")","ref":"Cryppo.html#encrypt_with_derived_key/4-example"},{"type":"function","title":"Cryppo.encryption_strategies/0","doc":"List available encryption strategies","ref":"Cryppo.html#encryption_strategies/0"},{"type":"function","title":"Cryppo.generate_encryption_key/1","doc":"Generate an encryption key for an encryption strategy\n\nThe generated encrypted key is marked as belonging to the encryption strategy.","ref":"Cryppo.html#generate_encryption_key/1"},{"type":"function","title":"Example - Cryppo.generate_encryption_key/1","doc":"iex> _encryption_key = Cryppo.generate_encryption_key(\"Aes256Gcm\")","ref":"Cryppo.html#generate_encryption_key/1-example"},{"type":"function","title":"Cryppo.load/1","doc":"Load various Cryppo data structures from their serialized forms\n\n3 Cryppo data structures have their own serialization formats:\n\n* `Cryppo.EncryptedData`\n* `Cryppo.EncryptedDataWithDerivedKey`\n* `Cryppo.RsaSignature`","ref":"Cryppo.html#load/1"},{"type":"function","title":"Examples - Cryppo.load/1","doc":"iex> s = \"Aes256Gcm.WSDb2AmsF7LFOxYb.QUAAAAACYWQABQAAAG5vbmUABWF0ABAAAAAAY9Ck6LzVGiMdiWFK6N5BawVpdgAMAAAAAG_Yxh-I0gGNYoFRigA=\"\n    iex> {:ok, %Cryppo.EncryptedData{}} = Cryppo.load(s)\n\n    iex> s = \"Aes256Gcm.wW4M_sv_kMx14cC6.QUAAAAACYWQABQAAAG5vbmUABWF0ABAAAAAA8Aq84t28sMT9FL8cz-TmMQVpdgAMAAAAANahbwbkfWo18YuCMgA=.Pbkdf2Hmac.SzAAAAAQaQAJUwAABWl2ABQAAAAAuQTqZLVFO49lI6Kx454ffYQ9VV0QbAAgAAAAAA==\"\n    iex> {:ok, %Cryppo.EncryptedDataWithDerivedKey{}} = Cryppo.load(s)\n\n    iex> s = \"Sign.Rsa4096.V4JbRzpkud-3cHCGqDwGjS3TmRto5Te0iSAtD7oIzsDa83McBDYpU_eeswVZF9AGEvoAEQOCwpqJ_PgbjHKT2nHgLysK-btG6Nxk_K2J7A6Uq15X5QrOgIKTzC00dj1tzAN73u9lsRPKIfwPyp_Mlb6FNs1LoB7OvAusit6QPm8iAwHo4nOWBBUf3hO9b3gsWJ92FxnBsCLYFQj_zv4mnLHj7pDNVtq9Kp4hK6bgcIH4FZtyDKDr6bXEtlCGLDIY10UqNLylkagI36Gyafm-HnD57vRxjgHIGEsd2XcwDJ8PqqrzSYNxl-RyWD3wq0nXE_1rYJ7k1AKLM5G1Hg8B2whqcXpQ52x3zVFCAjlU9GNhT6pdUBxQYw09va7fe2w517PrwwMe90MW87fj3G7dGEKT95cDLTx1d84ybIUFUJOGKY0FF4LL0E3UqWQ92kU4bh-DSTkNmgItX34fiBIOpQDbF238IkRYyFA8LfMPfL-0_dnto9sH0E3Umi41qFvpA2Nq8r57FF4vCOSkXYWVfyitOkY_URqMLxS57azwZRBehJYDtvbqmzaYEDceeLjkxDi--Y10LT4Cz2SGiU--YDJM66PZ3Cp74gvDpsWlohcwYmMib5LrjdtvLOAtOZhoLZyGeeX0lDnwOum7lFRpJd8UIrOlTvpBo48ep2bpmgA=.VmVyaMO8dHVuZyB2ZXJib3Rlbg==\"\n    iex> {:ok, %Cryppo.RsaSignature{}} = Cryppo.load(s)","ref":"Cryppo.html#load/1-examples"},{"type":"function","title":"Cryppo.serialize/1","doc":"Serialize various Cryppo data structures as a string\n\n3 Cryppo data structures have their own serialization formats:\n\n* `Cryppo.EncryptedData`\n* `Cryppo.EncryptedDataWithDerivedKey`\n* `Cryppo.RsaSignature`","ref":"Cryppo.html#serialize/1"},{"type":"function","title":"Examples - Cryppo.serialize/1","doc":"`Cryppo.EncryptedData`:\n\n    iex> {encrypted_data, _key} = Cryppo.encrypt(\"data to encrypt\", \"Aes256Gcm\")\n    iex> Cryppo.serialize(encrypted_data)\n\n`Cryppo.EncryptedDataWithDerivedKey`:\n\n    iex> \"data to encrypt\"\n    ...> |> Cryppo.encrypt_with_derived_key(\"Aes256Gcm\", \"Pbkdf2Hmac\", \"passphrase\")\n    ...> |> Cryppo.serialize()\n\n`Cryppo.RsaSignature`:\n\n    iex> private_key = Cryppo.generate_encryption_key(\"Rsa4096\")\n    iex> \"data to encrypt\"\n    ...> |> Cryppo.Rsa4096.sign(private_key)\n    ...> |> Cryppo.serialize()","ref":"Cryppo.html#serialize/1-examples"},{"type":"type","title":"Cryppo.encryption_strategy/0","doc":"Name of an encryption or derivation strategy\n\nUse `Cryppo.encryption_strategies/0` to get a list of encryption strategies.\nUse `Cryppo.derivation_strategies/0` to get a list of derivation strategies.","ref":"Cryppo.html#t:encryption_strategy/0"},{"type":"type","title":"Cryppo.encryption_strategy_module/0","doc":"Module of an encryption or derivation strategy","ref":"Cryppo.html#t:encryption_strategy_module/0"},{"type":"module","title":"Cryppo.DerivedKey","doc":"A struct for a derived encryption key and its derivation artefacts\n\nA `Cryppo.EncryptedData` struct may be marked as belonging to a certain key derivation strategy\nusing field `key_derivation_strategy` containing the module of the key derivation.\n\nA `Cryppo.DerivedKey` comes in 2 flavors:\n\n* With a derived encryption key. When used for encryption or decryption this key will be used\n* Without an encryption key. Encrypting or decrypting with this struct requires a passphrase to derive the key","ref":"Cryppo.DerivedKey.html"},{"type":"function","title":"Cryppo.DerivedKey.current_version/0","doc":"","ref":"Cryppo.DerivedKey.html#current_version/0"},{"type":"function","title":"Cryppo.DerivedKey.load_artefacts/1","doc":"","ref":"Cryppo.DerivedKey.html#load_artefacts/1"},{"type":"type","title":"Cryppo.DerivedKey.t/0","doc":"Struct `Cryppo.DerivedKey`\n\nA `Cryppo.DerivedKey` struct contains\n\n* `encryption_key` - `nil` or a `Cryppo.EncryptionKey`\n* `key_derivation_strategy` - module of the key derivation strategy\n* `salt` - salt used for key derivation\n* `iter` - number of iterations for key derivation\n* `length` - key length\n* `hash` - hash function for key derivation","ref":"Cryppo.DerivedKey.html#t:t/0"},{"type":"module","title":"Cryppo.EncryptedData","doc":"A struct for encrypted data and encryption artefacts\n\nAn `Cryppo.EncryptedData` struct may be marked as belonging to a certain encryption strategy\nusing field `encryption_strategy_module` containing the module of the encryption strategy.\n\nCan also contain encryption artefacts if they are part of the  encryption strategy.","ref":"Cryppo.EncryptedData.html"},{"type":"function","title":"Cryppo.EncryptedData.new/3","doc":"Initialize a struct with the module of an encryption strategy, a\nbinary with encrypted data, and encryption_artefacts.","ref":"Cryppo.EncryptedData.html#new/3"},{"type":"type","title":"Cryppo.EncryptedData.t/0","doc":"Struct `Cryppo.EncryptedData`\n\nA `Cryppo.EncryptedData` struct contains\n\n* `encrypted_data`: encrypted data\n* `encryption_strategy_module`: module of the encryption strategy to which the key belongs\n* `encryption_artefacts`: a map with encryption artefacts","ref":"Cryppo.EncryptedData.html#t:t/0"},{"type":"module","title":"Cryppo.EncryptedDataWithDerivedKey","doc":"A struct for a derived key and data encrypted with this derived key","ref":"Cryppo.EncryptedDataWithDerivedKey.html"},{"type":"type","title":"Cryppo.EncryptedDataWithDerivedKey.t/0","doc":"Struct `Cryppo.EncryptedData`\n\nA `Cryppo.EncryptedData` struct contains\n\n* `encrypted_data`: a `Cryppo.EncryptedData` struct\n* `derived_key`: a `Cryppo.DerivedKey` struct","ref":"Cryppo.EncryptedDataWithDerivedKey.html#t:t/0"},{"type":"module","title":"Cryppo.EncryptionArtefacts","doc":"A struct for encryption artefacts\n\nEach encryption strategy can use `Cryppo.EncryptionArtefacts` if it\nmakes sense for the underlying cipher.","ref":"Cryppo.EncryptionArtefacts.html"},{"type":"function","title":"Cryppo.EncryptionArtefacts.current_version/0","doc":"","ref":"Cryppo.EncryptionArtefacts.html#current_version/0"},{"type":"type","title":"Cryppo.EncryptionArtefacts.t/0","doc":"Struct `Cryppo.EncryptionArtefacts`","ref":"Cryppo.EncryptionArtefacts.html#t:t/0"},{"type":"module","title":"Cryppo.EncryptionKey","doc":"A struct to wrap an encryption key\n\nAn `EncryptionKey` struct may be marked as belonging to a certain encryption strategy\nusing field `encryption_strategy_module` containing the module of the encryption strategy.\n\nDepending on the encryption strategy the key can be a binary or a tuple.\n\nKeys must be guarded and protected, that is why the `Inspect` protocol has been\noverridden for this struct to disable pretty-printing.\n\nShould you need to access the key, access it via field `key`.","ref":"Cryppo.EncryptionKey.html"},{"type":"function","title":"Cryppo.EncryptionKey.new/1","doc":"Initialize a struct with an encryption key","ref":"Cryppo.EncryptionKey.html#new/1"},{"type":"function","title":"Cryppo.EncryptionKey.new/2","doc":"Initialize a struct with  an encryption key and the module of an encryption strategy","ref":"Cryppo.EncryptionKey.html#new/2"},{"type":"type","title":"Cryppo.EncryptionKey.binary_key/0","doc":"A binary key","ref":"Cryppo.EncryptionKey.html#t:binary_key/0"},{"type":"type","title":"Cryppo.EncryptionKey.internal_key/0","doc":"Encryption key\n\nAn encryption key may me a tuple or a binary depending on the encryption strategy","ref":"Cryppo.EncryptionKey.html#t:internal_key/0"},{"type":"type","title":"Cryppo.EncryptionKey.t/0","doc":"Struct EncryptionKey\n\nA `Cryppo.EncryptionKey` struct contains\n\n* `key`: the key itself\n* `encryption_strategy_module`: module of the encryption strategy to which the key belongs","ref":"Cryppo.EncryptionKey.html#t:t/0"},{"type":"module","title":"Cryppo.Rsa4096","doc":"Encryption strategy RSA with 4096-bit keys and some RSA-specific functions\n\nFor encryption and decryption please use functions in module `Cryppo`.\nThis module also contains logic for PEMs, singing and verification.","ref":"Cryppo.Rsa4096.html"},{"type":"function","title":"Cryppo.Rsa4096.from_pem/1","doc":"Loads and initializes a `Cryppo.EncryptionKey` struct from a string with a PEM.","ref":"Cryppo.Rsa4096.html#from_pem/1"},{"type":"function","title":"Examples - Cryppo.Rsa4096.from_pem/1","doc":"iex> pem = \"-----BEGIN RSA PRIVATE KEY-----\\n\"  \n    ...>       \"MIICWwIBAAKBgQDKCUh7F4p5btzcSLBaToHvD3rCZX4fMaDtjkN5TwmC3/6iQzD5\\n\"  \n    ...>       \"tn396BzDTdQ16HuuZ+eN+KQSa1QWr2h1DB13nVP+moeyLVC8BShiM3NBRn77r7Lr\\n\"  \n    ...>       \"sWooM3mwnSvMPWWnBj1c+0tbO7zfur5wQdzBl66HrHgHt+Bz6f+dDj+aVwIDAQAB\\n\"  \n    ...>       \"AoGAMHh3rihgrW9+h07dGF1baOoyzm6hCoTSkguefn0K0B5DLdSm7FHu+jp0pBqI\\n\"  \n    ...>       \"/gHvolEFSZdMbarYOrUMf4BPlRSarCjjxf/beV4Pj/UQrCkDmNBBVJp33Sy8HEdb\\n\"  \n    ...>       \"Wrzk+k8NcAS1UR4R6EW9JrUz0mMwX6CsvG2zZMbpS/Q9KXkCQQDwmCXjOTPQ+bxW\\n\"  \n    ...>       \"K4gndHnXD5QkKNcTdFq64ef23R6AY0XEGkiRLDXZZA09hDIACgSSfk1Qbo0SJSvU\\n\"  \n    ...>       \"TAR8A6clAkEA1vkWJ5qUo+xuIZB+2604LRco1GYAj5/fZ2kvUMjbOdCFgFaDVzJY\\n\"  \n    ...>       \"X2pzLkk7RZNgPvXcRAgX7FlWmm4jwZzQywJARrHeSCMRx7DqF0PZUQaXmorYU7uw\\n\"  \n    ...>       \"XuYMluc0WsRkZwNEh7fVZNrhw8vzXAUREBPhfg4gt6aUSyWi+FGR68LDBQJAC55O\\n\"  \n    ...>       \"ujk6i1l94kaC9LB59sXnqQMSSLDlTBt9OSqB3rAMZxFF6/KGoDGKpBfFIk+CxiRX\\n\"  \n    ...>       \"kT22vUleyt3lBNPK3QJAEr56asvREcIDFkbs7Ebjev4U1PL58w78ipp49Ti5FiwH\\n\"  \n    ...>       \"vR9vuGcUcIDcWKOl05t4D35F5A/DskP6dGYA1cuWNg==\\n\"  \n    ...>       \"-----END RSA PRIVATE KEY-----\\n\\n\"\n    ...> {:ok, _encryption_key} = Cryppo.Rsa4096.from_pem(pem)","ref":"Cryppo.Rsa4096.html#from_pem/1-examples"},{"type":"function","title":"Cryppo.Rsa4096.private_key_to_public_key/1","doc":"Extracts a public key from a private key\n\nExtracts a public key from a `Cryppo.EncryptionKey` struct with an RSA private key or from an\nRSA private key in the native Erlang type `t:rsa_private_key/0`","ref":"Cryppo.Rsa4096.html#private_key_to_public_key/1"},{"type":"function","title":"Examples - Cryppo.Rsa4096.private_key_to_public_key/1","doc":"With a `Cryppo.EncryptionKey` struct:\n\n    iex> public_key = \"Rsa4096\"\n    ...> |> Cryppo.generate_encryption_key()\n    ...> |> Cryppo.Rsa4096.private_key_to_public_key()\n    ...> elem(public_key, 0)\n    :RSAPublicKey\n\nWith a native Erlang key:\n\n    iex> public_key = {:rsa, 4_096, 65_537}\n    ...> |> :public_key.generate_key()\n    ...> |> Cryppo.Rsa4096.private_key_to_public_key()\n    ...> elem(public_key, 0)\n    :RSAPublicKey","ref":"Cryppo.Rsa4096.html#private_key_to_public_key/1-examples"},{"type":"function","title":"Cryppo.Rsa4096.sign/2","doc":"Signs data with a private key\n\nThe private key can be one of the following:\n\n* a `Cryppo.EncryptionKey` struct\n* a private key as native Erlang type `t:rsa_private_key/0`\n* a PEM with a private RSA key","ref":"Cryppo.Rsa4096.html#sign/2"},{"type":"function","title":"Examples - Cryppo.Rsa4096.sign/2","doc":"With a `Cryppo.EncryptionKey` struct:\n\n    iex> encryption_key = Cryppo.generate_encryption_key(\"Rsa4096\")\n    iex> _signature = %Cryppo.RsaSignature{} = Cryppo.Rsa4096.sign(\"data to sign\", encryption_key)\n\n\nWith a private key as native Erlang type `t:rsa_private_key/0`\n\n    iex> private_key = :public_key.generate_key({:rsa, 4_096, 65_537})\n    iex> _signature = %Cryppo.RsaSignature{} = Cryppo.Rsa4096.sign(\"data to sign\", private_key)\n\nWith a PEM\n\n    iex> pem = \"-----BEGIN RSA PRIVATE KEY-----\\n\"  \n    ...>       \"MIICWwIBAAKBgQDKCUh7F4p5btzcSLBaToHvD3rCZX4fMaDtjkN5TwmC3/6iQzD5\\n\"  \n    ...>       \"tn396BzDTdQ16HuuZ+eN+KQSa1QWr2h1DB13nVP+moeyLVC8BShiM3NBRn77r7Lr\\n\"  \n    ...>       \"sWooM3mwnSvMPWWnBj1c+0tbO7zfur5wQdzBl66HrHgHt+Bz6f+dDj+aVwIDAQAB\\n\"  \n    ...>       \"AoGAMHh3rihgrW9+h07dGF1baOoyzm6hCoTSkguefn0K0B5DLdSm7FHu+jp0pBqI\\n\"  \n    ...>       \"/gHvolEFSZdMbarYOrUMf4BPlRSarCjjxf/beV4Pj/UQrCkDmNBBVJp33Sy8HEdb\\n\"  \n    ...>       \"Wrzk+k8NcAS1UR4R6EW9JrUz0mMwX6CsvG2zZMbpS/Q9KXkCQQDwmCXjOTPQ+bxW\\n\"  \n    ...>       \"K4gndHnXD5QkKNcTdFq64ef23R6AY0XEGkiRLDXZZA09hDIACgSSfk1Qbo0SJSvU\\n\"  \n    ...>       \"TAR8A6clAkEA1vkWJ5qUo+xuIZB+2604LRco1GYAj5/fZ2kvUMjbOdCFgFaDVzJY\\n\"  \n    ...>       \"X2pzLkk7RZNgPvXcRAgX7FlWmm4jwZzQywJARrHeSCMRx7DqF0PZUQaXmorYU7uw\\n\"  \n    ...>       \"XuYMluc0WsRkZwNEh7fVZNrhw8vzXAUREBPhfg4gt6aUSyWi+FGR68LDBQJAC55O\\n\"  \n    ...>       \"ujk6i1l94kaC9LB59sXnqQMSSLDlTBt9OSqB3rAMZxFF6/KGoDGKpBfFIk+CxiRX\\n\"  \n    ...>       \"kT22vUleyt3lBNPK3QJAEr56asvREcIDFkbs7Ebjev4U1PL58w78ipp49Ti5FiwH\\n\"  \n    ...>       \"vR9vuGcUcIDcWKOl05t4D35F5A/DskP6dGYA1cuWNg==\\n\"  \n    ...>       \"-----END RSA PRIVATE KEY-----\\n\\n\"\n    ...> _signature = %Cryppo.RsaSignature{} = Cryppo.Rsa4096.sign(\"data to sign\", pem)","ref":"Cryppo.Rsa4096.html#sign/2-examples"},{"type":"function","title":"Cryppo.Rsa4096.to_pem/1","doc":"Converts an RSA key to PEM format.\n\nCan convert\n\n* a `Cryppo.EncryptionKey` struct\n* a public key as native Erlang type `t:rsa_public_key/0`\n* a private key as native Erlang type `t:rsa_private_key/0`","ref":"Cryppo.Rsa4096.html#to_pem/1"},{"type":"function","title":"Examples - Cryppo.Rsa4096.to_pem/1","doc":"With a `Cryppo.EncryptionKey` struct\n\n    iex> \"Rsa4096\" |> Cryppo.generate_encryption_key() |> Cryppo.Rsa4096.to_pem()\n\n\nWith a public key as native Erlang type `t:rsa_public_key/0`\n\n    iex> \"Rsa4096\"\n    ...> |> Cryppo.generate_encryption_key()\n    ...> |> Cryppo.Rsa4096.private_key_to_public_key()\n    ...> |> Cryppo.Rsa4096.to_pem()\n\nWith a private key as native Erlang type `t:rsa_private_key/0`\n\n    iex> encryption_key = Cryppo.generate_encryption_key(\"Rsa4096\")\n    iex> Cryppo.Rsa4096.to_pem(encryption_key.key)","ref":"Cryppo.Rsa4096.html#to_pem/1-examples"},{"type":"function","title":"Cryppo.Rsa4096.verify/2","doc":"Verifies an RSA signature with a public key\n\nThe key for verification can be pretty much any format and type, private keys are also accepted:\n\n* native Erlang types `t:rsa_private_key/0` and `t:rsa_public_key/0`\n* `Cryppo.EncryptionKey` structs\n* PEMs","ref":"Cryppo.Rsa4096.html#verify/2"},{"type":"function","title":"Examples - Cryppo.Rsa4096.verify/2","doc":"With a public key in the Erlang format:\n\n    iex> encryption_key = Cryppo.generate_encryption_key(\"Rsa4096\")\n    iex> signature = Cryppo.Rsa4096.sign(\"data to sign\", encryption_key)\n    iex> public_key = Cryppo.Rsa4096.private_key_to_public_key(encryption_key)\n    iex> Cryppo.Rsa4096.verify(signature, public_key)\n    true\n\nWith a private key in the Erlang format:\n\n    iex> encryption_key = Cryppo.generate_encryption_key(\"Rsa4096\")\n    iex> signature = Cryppo.Rsa4096.sign(\"data to sign\", encryption_key)\n    iex> Cryppo.Rsa4096.verify(signature, encryption_key.key)\n    true\n\nWith a `Cryppo.EncryptionKey` struct:\n\n    iex> encryption_key = Cryppo.generate_encryption_key(\"Rsa4096\")\n    iex> signature = Cryppo.Rsa4096.sign(\"data to sign\", encryption_key)\n    iex> Cryppo.Rsa4096.verify(signature, encryption_key)\n    true\n\nWith a PEM\n\n    iex> pem_with_private_key = \"-----BEGIN RSA PRIVATE KEY-----\\n\"  \n    ...>       \"MIICWwIBAAKBgQDKCUh7F4p5btzcSLBaToHvD3rCZX4fMaDtjkN5TwmC3/6iQzD5\\n\"  \n    ...>       \"tn396BzDTdQ16HuuZ+eN+KQSa1QWr2h1DB13nVP+moeyLVC8BShiM3NBRn77r7Lr\\n\"  \n    ...>       \"sWooM3mwnSvMPWWnBj1c+0tbO7zfur5wQdzBl66HrHgHt+Bz6f+dDj+aVwIDAQAB\\n\"  \n    ...>       \"AoGAMHh3rihgrW9+h07dGF1baOoyzm6hCoTSkguefn0K0B5DLdSm7FHu+jp0pBqI\\n\"  \n    ...>       \"/gHvolEFSZdMbarYOrUMf4BPlRSarCjjxf/beV4Pj/UQrCkDmNBBVJp33Sy8HEdb\\n\"  \n    ...>       \"Wrzk+k8NcAS1UR4R6EW9JrUz0mMwX6CsvG2zZMbpS/Q9KXkCQQDwmCXjOTPQ+bxW\\n\"  \n    ...>       \"K4gndHnXD5QkKNcTdFq64ef23R6AY0XEGkiRLDXZZA09hDIACgSSfk1Qbo0SJSvU\\n\"  \n    ...>       \"TAR8A6clAkEA1vkWJ5qUo+xuIZB+2604LRco1GYAj5/fZ2kvUMjbOdCFgFaDVzJY\\n\"  \n    ...>       \"X2pzLkk7RZNgPvXcRAgX7FlWmm4jwZzQywJARrHeSCMRx7DqF0PZUQaXmorYU7uw\\n\"  \n    ...>       \"XuYMluc0WsRkZwNEh7fVZNrhw8vzXAUREBPhfg4gt6aUSyWi+FGR68LDBQJAC55O\\n\"  \n    ...>       \"ujk6i1l94kaC9LB59sXnqQMSSLDlTBt9OSqB3rAMZxFF6/KGoDGKpBfFIk+CxiRX\\n\"  \n    ...>       \"kT22vUleyt3lBNPK3QJAEr56asvREcIDFkbs7Ebjev4U1PL58w78ipp49Ti5FiwH\\n\"  \n    ...>       \"vR9vuGcUcIDcWKOl05t4D35F5A/DskP6dGYA1cuWNg==\\n\"  \n    ...>       \"-----END RSA PRIVATE KEY-----\\n\\n\"\n    ...> signature = Cryppo.Rsa4096.sign(\"data to sign\", pem_with_private_key)\n    ...> {:ok, key} = Cryppo.Rsa4096.from_pem(pem_with_private_key)\n    ...> {:ok, pem_with_public_key} = key\n    ...> |> Cryppo.Rsa4096.private_key_to_public_key()\n    ...> |> Cryppo.Rsa4096.to_pem()\n    ...> Cryppo.Rsa4096.verify(signature, pem_with_public_key)\n    true","ref":"Cryppo.Rsa4096.html#verify/2-examples"},{"type":"type","title":"Cryppo.Rsa4096.pem/0","doc":"RSA keys in PEM format","ref":"Cryppo.Rsa4096.html#t:pem/0"},{"type":"type","title":"Cryppo.Rsa4096.rsa_private_key/0","doc":"Erlang type for RSA private keys\n\nThe native Erlang type for RSA private keys in module [`public_key`](https://erlang.org/doc/man/public_key.html)\nare Erlang records visible from Elixir as tuples with 11 terms the first term being atom `:RSAPrivateKey`","ref":"Cryppo.Rsa4096.html#t:rsa_private_key/0"},{"type":"type","title":"Cryppo.Rsa4096.rsa_public_key/0","doc":"Erlang type for RSA public keys\n\nThe native Erlang type for RSA public keys in module [`public_key`](https://erlang.org/doc/man/public_key.html)\nare Erlang records visible from Elixir as tuples with 3 terms the first term being atom `:RSAPublicKey`","ref":"Cryppo.Rsa4096.html#t:rsa_public_key/0"},{"type":"module","title":"Cryppo.RsaSignature","doc":"A struct for an RSA singature and the signed data","ref":"Cryppo.RsaSignature.html"},{"type":"type","title":"Cryppo.RsaSignature.t/0","doc":"Struct `Cryppo.RsaSignature`\n\nA `Cryppo.RsaSignature` struct contains\n\n* `signature`: RSA signature\n* `data`: signed data","ref":"Cryppo.RsaSignature.html#t:t/0"},{"type":"extras","title":"README","doc":"[![Open in GitHub Codespaces](https://github.com/codespaces/badge.svg)](https://github.com/codespaces/new?hide_repo_select=true&ref=master&repo=249279012)\n\n[![Github Actions](https://github.com/leikind/cryppo_ex/actions/workflows/ci.yml/badge.svg)](https://github.com/leikind/cryppo_ex/actions)\n\n# CryppoEx\n\nCryppoEx is a cryptographic library that enables you to encrypt and decrypt data.\nCryppoEx combines very different ciphers under one simplified API, and a set of serialization formats.\n\nCryppoEx is an Elixir port of [Cryppo in Ruby](https://github.com/Meeco/cryppo), [Cryppo in Dart](https://pub.dev/packages/cryppo),\nand [Cryppo-js](https://github.com/Meeco/cryppo-js) used for [the Meeco platform](https://www.meeco.me/platform).\n\nCryppoEx uses Erlang modules [crypto](https://erlang.org/doc/man/crypto.html)\nand [public_key](https://erlang.org/doc/man/public_key.html) under the hood.","ref":"readme.html"},{"type":"extras","title":"Installation - README","doc":"Add `cryppo_ex` as a dependency in your `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:cryppo_ex, \"~> 0.1\"}\n  ]\nend\n```\n\nRun `mix deps.get` to fetch CryppoEx.","ref":"readme.html#installation"},{"type":"extras","title":"Encrypt and decrypt data using a derived key - README","doc":"When encrypting data with a user-generated passphrase or password, use function `Cryppo.encrypt_with_derived_key/4`.\n\nThe data will be encrypted with a cryptographically secure key that is derived from the passphrase:\n\n```elixir\nencryption_strategy = \"Aes256Gcm\"\nkey_derivation_strategy = \"Pbkdf2Hmac\"\npassphrase = \"MyPassword!!\"\ndata = \"some data to encrypt\"\n\nencrypted_data = Cryppo.encrypt_with_derived_key(data, encryption_strategy, key_derivation_strategy, passphrase)\n```\n\nYou can list all available encryption strategies with `Cryppo.encryption_strategies/0`,\nand all key derivation strategies with `Cryppo.derivation_strategies/0`.\n\nThe encryption process will return a `Cryppo.EncryptedDataWithDerivedKey` struct that contains all the encryption\nartefacts necessary to decrypt the encrypted data.\nThis structure can be serialized as a string using function `Cryppo.serialize/1`.\nThe serialized payload can be stored directly in a data store.\n\n```elixir\nserialized = Cryppo.serialize(encrypted_data)\n```\n\nThe serialized payload can later be loaded by using `Cryppo.load/1` and decrypted with\n`Cryppo.decrypt_with_derived_key/2` and the passphrase:\n\n```elixir\n{:ok, encrypted} = Cryppo.load(serialized)\n{:ok, decrypted, _encryption_key} = Cryppo.decrypt_with_derived_key(encrypted, passphrase)\n\nIO.inspect(decrypted) #=> \"some data to encrypt\"\n```","ref":"readme.html#encrypt-and-decrypt-data-using-a-derived-key"},{"type":"extras","title":"Encrypt and decrypt data using a generated cryptographic key - README","doc":"You can also encrypt using your own generated key using functions\nthe `Cryppo.generate_encryption_key/1` and `Cryppo.encrypt/3`:\n\n```elixir\nencryption_strategy = \"Aes256Gcm\"\ndata = \"some data to encrypt\"\n\nencryption_key = Cryppo.generate_encryption_key(encryption_strategy)\nencrypted = Cryppo.encrypt(data, encryption_strategy, encryption_key)\n```\n\nThe encryption process will return an `Cryppo.EncryptedData` struct that contains all the encryption\nartefacts necessary to decrypt the encrypted data.\n\nIt is also possible to generate a key and encrypt data in one go with `Cryppo.encrypt/2`:\n\n```elixir\n{encrypted_data, encryption_key} = Cryppo.encrypt(data, encryption_strategy)\n```\n\n`Cryppo.EncryptedData` structs can be serialized as a string using function `Cryppo.serialize/1`.\nThe serialized payload can be stored directly in a data store.\n\n```elixir\nserialized = Cryppo.serialize(encrypted_data)\n```\n\nThe serialized payload can later be loaded by using `Cryppo.load/1` and decrypted with\n`Cryppo.decrypt/2` and the passphrase:\n\n```elixir\n{:ok, encrypted} = Cryppo.load(serialized)\n{:ok, decrypted} = Cryppo.decrypt(encrypted, encryption_key)\n\nCryppo.inspect(decrypted) #=> \"some data to encrypt\"\n```","ref":"readme.html#encrypt-and-decrypt-data-using-a-generated-cryptographic-key"},{"type":"extras","title":"Signing and verification - README","doc":"For authentication purposes, a sender can sign a message with their private key,\nand a recipient can verify this signature using the sender's public key.\n\n```elixir\nprivate_key = Cryppo.generate_encryption_key(\"Rsa4096\")\n\nrsa_signature = Cryppo.Rsa4096.sign(\"data to verify\", private_key)\nserialized = Cryppo.serialize(rsa_signature)\n\n{:ok, signature} = Cryppo.load(serialized)\nCryppo.Rsa4096.verify(signature, Cryppo.Rsa4096.private_key_to_public_key(private_key)) # => true\n```","ref":"readme.html#signing-and-verification"},{"type":"extras","title":"Encryption Strategies - README","doc":"#","ref":"readme.html#encryption-strategies"},{"type":"extras","title":"Aes256Gcm - README","doc":"Aes256Gcm was chosen because it provides authenticated encryption.\nAn error will be raised if an incorrect value, such as the encryption key, were used during decryption.\nThis means you can always be sure that the decrypted data is the same as the data that was originally encrypted.","ref":"readme.html#aes256gcm"},{"type":"extras","title":"Key Derivation Strategies - README","doc":"#","ref":"readme.html#key-derivation-strategies"},{"type":"extras","title":"Pbkdf2Hmac - README","doc":"Pbkdf2Hmac generates cryptographically secure keys from potentially insecure sources such as user-generated passwords.\n\nThe derived key is cryptographically secure such that brute force attacks directly on the encrypted data is infeasible.\nThe amount of computational effort required to complete the operation can be tweaked.\nThis ensures that brute force attacks on the password encrypted data.","ref":"readme.html#pbkdf2hmac"},{"type":"extras","title":"Command Line Interface - README","doc":"CryppoEx exposes its functionality via a command line interface.\n\nIn order to use the CryppoEx CLI, you need to build an\n[escript](https://hexdocs.pm/mix/master/Mix.Tasks.Escript.Build.html) executable with the following command:\n\n```sh\nMIX_ENV=prod mix escript.build\n```\n\nThe generated executable can run on any machine that has Erlang installed and does not require Elixir to be installed.\n\n\n### `cryppo genkey`\n\nGenerate a new (random) encryption key - printed as base64 encoded\n\n```\nUSAGE\n  cryppo genkey -s [ENCRYPTION_STRATEGY]\n\nOPTIONS\n  -s, --strategy=strategy  encryption strategy (defaults to Aes256Gcm)\n\nEXAMPLES\n  cryppo genkey\n  cryppo genkey --strategy=Aes256Gcm\n```\n\n### `cryppo genkeypair`\n\nGenerate a new RSA key pair, writing the private and public keys to files.\n\n```\nUSAGE\n  cryppo genkeypair -p [PRIVATE_KEY_FILE] -P [PUBLIC_KEY_FILE]\n\nOPTIONS\n  -p, --privateKeyOut=privateKeyOut  (required) Private key output path\n  -P, --publicKeyOut=publicKeyOut    (required) Public key output path\n\nEXAMPLE\n  cryppo genkeypair -p private.pem -P public.pem\n```\n\n### `cryppo encrypt`\n\nEncrypt data with a generated key\n\n```\nUSAGE\n  cryppo encrypt -v [DATA] -k [KEY] -s [ENCRYPTION_STRATEGY]\n  cryppo encrypt -v [DATA] -P [PUBLIC_KEY_FILE]\n\nOPTIONS\n  -v, --value=value                  (required) value to encrypt\n  -s, --strategy=strategy            encryption strategy (defaults to Aes256Gcm)\n  -k, --key=key                      base64 encoded data encryption key\n  -P, --publicKeyFile=publicKeyFile  public key file (if encrypting with RSA)\n\nEXAMPLES\n  encrypt -v \"hello world\" -k vm8CjugMda2zdjsI9W25nH-CY-84DDYoBxTFLwfKLDk= -s Aes256Gcm\n  encrypt -v \"hello world\" -P public.pem\n```\n\n### `cryppo decrypt`\n\nDecrypt a serialized encrypted value\n\n```\nUSAGE\n  cryppo decrypt -e [ENCRYPTED_DATA] -k [KEY] -s [ENCRYPTION_STRATEGY]\n  cryppo decrypt -e [ENCRYPTED_DATA] -p [PRIVATE_KEY_FILE]\n\nOPTIONS\n  -e, --encrypted=encrypted            (required) serialized encrypted value\n  -s, --strategy=strategy              encryption strategy (defaults to Aes256Gcm)\n  -k, --key=key                        base64 encoded data encryption key\n  -p, --privateKeyFile=privateKeyFile  private key file (if encrypting with RSA)\n\nEXAMPLES\n  cryppo decrypt -e\n  \"Aes256Gcm.gSAByGMq4edzM0U=.LS0tCml2OiAhYmluYXJ5IHwtCiAgaW1QL09qMWZ6eWw0cmwwSgphdDogIWJpbmFyeSB8LQogIE5SbjZUQXJ2bitNS1\n  Z5M0FpZEpmWlE9PQphZDogbm9uZQo=\" -k vm8CjugMda2zdjsI9W25nH-CY-84DDYoBxTFLwfKLDk=\n\n  cryppo decrypt -e \"Rsa4096.bJjV2g_RBZKeyqBr-dSjPAc3qtkTgd0=.LS0tCnt9Cg==\" -p private.pem\n```\n\n### `cryppo encrypt-der`\n\nEncrypt data with a derived key\n\n```\nUSAGE\ncryppo encrypt-der -v [DATA] -w [PASSWORD] -s [ENCRYPTION_STRATEGY] -d [DERIVATION_STRATEGY]\n\nOPTIONS\n  -v, --value=value                  (required) value to encrypt\n  -w, --password=password            (required) password for key derivation\n  -s, --strategy=strategy            encryption strategy (defaults to Aes256Gcm)\n  -d, --derivation-strategy=strategy derivation strategy (defaults to Pbkdf2Hmac)\n\nEXAMPLES\n  cryppo encrypt-der -v \"hello world\" -w \"secret phrase\" -s Aes256Gcm -d Pbkdf2Hmac\n\n  cryppo encrypt-der -v \"hello world\" -w \"secret phrase\"\n```\n\n### `cryppo decrypt-der`\n\nDecrypt a serialized encrypted value with a derived key\n\n```\nUSAGE\n  cryppo decrypt-der -e [ENCRYPTED_DATA] -p [PASSPHRASE]\n\nOPTIONS\n  -e, --encrypted=encrypted          (required) serialized encrypted value\n  -p, --passphrase=passphrase        (required) passphrase for key derivation\n\nEXAMPLES\n  cryppo decrypt-der -p \"secret phrase\"  \\\n  -e \"Aes256Gcm.e-IJT9E8ew3wlz8=.LS0tCmFkOiBub25lCmF0OiAhIWJpbmFyeSB8LQogIHpTRzQzbVhlSFBsR3ZQQVZoNTVJQUE9PQppdjogISFiaW5hcnkgfC0KICBMU2NDNmVCZ2wrUCtuUkpaCg==.Pbkdf2Hmac.LS0tCidpJzogMjEzMjIKJ2l2JzogISFiaW5hcnkgfC0KICBzTmlGT21xWEg5b1piNzRNVElCcGxvNHlHV2M9CidsJzogMzIK\"\n```\n\n### `cryppo sign FILE DESTINATION`\n\nSign a file with an RSA private key and write the signed contents to a new file\n\n```\nUSAGE\n  cryppo sign -p [PRIVATE_KEY_FILE] FILE DESTINATION\n\nARGUMENTS\n  FILE         File to sign\n  DESTINATION  file to write the resulting signed content to\n\nOPTIONS\n  -p, --privateKeyFile=privateKeyFile  (required) path to the private key file\n\nEXAMPLE\n  cryppo sign -p private.pem my_file.txt my_file.signed.txt\n```\n\n### `cryppo verify FILE DESTINATION`\n\nVerify an RSA signed file and write the contents to another file.\n\n```\nUSAGE\n  cryppo verify -P [PUBLIC_KEY_FILE] FILE DESTINATION\n\nARGUMENTS\n  FILE         Signed file contents to verify\n  DESTINATION  File to write the resulting verified content to\n\nOPTIONS\n  -P, --publicKeyFile=publicKeyFile  (required) path to the public key file\n\nEXAMPLE\n  cryppo verify -P public.pem my_file.signed.txt my_file.txt\n```","ref":"readme.html#command-line-interface"}],"content_type":"text/markdown"}